﻿using System.Net.Sockets;
using System.Text;
using System.Threading;
using DSCLogic.Commands;

namespace DSCLogic.Communications
{

    public class Communicator
    {

        private readonly Reciever _reciever;

        private readonly TcpClient _clientSocket = new TcpClient();
        private readonly string _envisaLinkdTpiServer;
        private Thread _listenThread;
        private Thread _pollThread;
        public Communicator(string envisaLinkdTpiServer)
        {
            _envisaLinkdTpiServer = envisaLinkdTpiServer;
            _reciever = new Reciever(_clientSocket);
        }

        public void Connect()
        {
            _clientSocket.Connect(_envisaLinkdTpiServer, 4025);
            _listenThread = new Thread(ListenForData);
            _listenThread.Start(); // start listening thread.
        }

        public void EnablePoll()
        {
            if (_pollThread == null || !_pollThread.IsAlive)
            { // only start if it aint started.
                _pollThread = new Thread(Poll);
                _pollThread.Start(); // start listening thread.
            }
        }

        public void ExecuteRequest(TpiRequest tpiRequest)
        {
            var data = Encoding.ASCII.GetBytes(tpiRequest.Data);
            var serverStream = _clientSocket.GetStream();
            serverStream.Write(data, 0, data.Length);
        }

        public void Subscribe(ResponseEventHandler responseEventHandler)
        {
            _reciever.Subscribe(responseEventHandler);
        }
        public void UnSubscribe(ResponseEventHandler responseEventHandler)
        {
            _reciever.UnSubscribe(responseEventHandler);
        }



        private void ListenForData()
        {
            try
            {
                _reciever.Run();
            }
            catch
            {
                //nothing, check shutdown flag.
            }

            if (!_reciever.Shutdown) // if not in shutdown mode, try to restart
            {
                Connect(); // restart
            }

        }
        private void Poll()
        {
            var r = new TpiRequest(RequestCommand.Poll);
            while (!_reciever.Shutdown)
            {
                ExecuteRequest(r);
                try
                {
                    Thread.Sleep(60 * 1000 * 2); // sleep 2 minutes before next poll
                }
                catch
                {
                    //nothing, check shutdown flag.
                }
            }
        }

        public void Quit()
        {
            _reciever.Shutdown = true;
            _pollThread.Interrupt();
            _listenThread.Interrupt();
        }
    }


}
