﻿using System;
using DSCLogic.Commands;
using DSCLogic.Communications;
using DSCLogic.Exceptions;

namespace DSCLogic
{
    public class Example
    {
        static void MainMethod()
        {
            //init with name (or IP) to dslink and the password
            //username is never used only the password.
            new Example("dslink.home.local", "password");  
        }

        private readonly Communicator _communicator;
        private readonly string _password;
        public Example(string serverName, string password)
        {
            _password = password;
            _communicator = new Communicator(serverName); // init the communicator.
            _communicator.Subscribe(new LoginHandler(ContinueWithLogin)); // add a loginhandler
            _communicator.Connect(); // tell it to connect to dslink.

            // loginhandler shold now get an LoginInteraction event with data == 3, which is a password request from dslink
            
        }

        private void ContinueWithLogin()
        {
            Console.WriteLine("Sending key");
            _communicator.ExecuteRequest(new TpiRequest(RequestCommand.NetworkLogin, _password));
            Console.WriteLine("Enabling timestamp");
            _communicator.ExecuteRequest(new TpiRequest(RequestCommand.TimeStampControl, "1")); // enable timestamp
            Console.WriteLine("Starting poll");
            _communicator.EnablePoll(); // start poll (keep alive) thread

            Console.WriteLine("Subscribing with ZoneHandler");
            _communicator.Subscribe(new CommandHandler());
            _communicator.ExecuteRequest(new TpiRequest(RequestCommand.StatusReport)); 
        }

        //inner example class for handling password request
        //it also unsubscribes itself after the password is sent.
        class LoginHandler : ResponseEventHandler
        {
            private const string IncorrectPassword = "0";
            private const string PasswordCorrect = "1";
            private const string TimeOut = "2";
            private const string PasswordRequest = "3";
            private readonly Action _action;
            public LoginHandler(Action continueWithLogin)
            {
                _action = continueWithLogin;
            }

            public override void EventOccured(TpiResponse response, Reciever r)
            {
                if (ResponseCommand.LoginInteraction == response.Command)
                {

                    if (response.Data.Equals(PasswordRequest))
                    {
                        Console.WriteLine("Password request recived");
                        _action.Invoke();

                        //possible to remove this now, but keeping it active, in case the connection resets
                        //and another password request is recieved
                        //r.UnSubscribe(this);
                    }
                    else if (response.Data.Equals(PasswordCorrect))
                    {
                        Console.WriteLine("Password accepted");
                    }
                    else if (response.Data.Equals(IncorrectPassword))
                    {
                        Console.WriteLine("Incorrect Password");
                        throw new LoginException("Incorrect Password");
                    }
                    else if (response.Data.Equals(TimeOut))
                    {
                        Console.WriteLine("Password accepted");
                        throw new LoginException("Password not sent within 10 seconds");
                    }
                }
            }
        }
        //inner example class to display all events recieved from dslink
        //outputs everything to the console.
        class CommandHandler : ResponseEventHandler
        {
            public override void EventOccured(TpiResponse response, Reciever r)
            {
                //write date from dlsink <tab> command, data
                Console.WriteLine(response.Date + "\tCommand: " + response.Command + ", Data: " + response.Data);
            }
        }


    }
}
